package net.sf.doolin.util;

import net.sf.jstring.LocalizableException;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.OutputStreamWriter;
import java.util.Arrays;
import java.util.Random;

/**
 * Utility methods for dealing with passwords.
 * 
 * @author Damien Coraboeuf
 */
public class PasswordUtils {

	/**
	 * Blank password
	 */
	private final static String blankPassword = digestPassword(null);

	/**
	 * Random generator
	 */
	private final static Random random = new Random(System.currentTimeMillis());

	/**
	 * Decodes a password from Base64
	 * 
	 * @param s
	 *            Encoded password
	 * @return Decoded password
	 */
	public static char[] decodePassword(String s) {
		try {
			return CodecUtils.decodeCharsBase64(s);
		} catch (LocalizableException ex) {
			throw ex;
		} catch (Exception ex) {
			throw new LocalizableException(
					StringCodes.STRING_ERROR_CANNOT_DECODE_PASSWORD, ex);
		}
	}

	/**
	 * Computes an encoded digest for a password
	 * 
	 * @param password
	 *            Password to encode
	 * @return Digest for the password
	 */
	public static String digestPassword(char[] password) {
		if (password == null || password.length == 0) {
			return "";
		}
		byte[] bytes = null;
		try {
			bytes = passwordToBytes(password);
			// Calculates the digest
			String digest = DigestUtils.md5Hex(bytes);
			// Ok
			return digest;
		} finally {
			// Clears all arrays
			if (bytes != null) {
				Arrays.fill(bytes, (byte) 0);
			}
			Arrays.fill(password, (char) 0);
		}
	}

	/**
	 * Encodes a password using Base64
	 * 
	 * @param password
	 *            Password to encode
	 * @return Encoded password
	 */
	public static String encodePassword(char[] password) {
		try {
			return CodecUtils.encodeCharsBase64(password);
		} catch (LocalizableException ex) {
			throw ex;
		} catch (Exception ex) {
			throw new LocalizableException(
					StringCodes.STRING_ERROR_CANNOT_ENCODE_PASSWORD, ex);
		}
	}

	/**
	 * Generates an arbitrary token with <code>nChars</code> character length.
	 * 
	 * @param nChars
	 *            Number of characters to generate
	 * @return Resulting string
	 */
	public static String generateToken(int nChars) {
		// base 64: 3 bytes = 4 chars
		int cb = (nChars + 3) / 4 * 3;
		byte[] ab = new byte[cb];
		random.nextBytes(ab);
		// encode the random bytes as a base-64 value of the desired length;
		// also: replace the special character '/' with the legal '$'
		return CodecUtils.encodeBase64(ab).replace('/', '$');
	}

	/**
	 * Is the password blank ? This method uses the password with its encoded
	 * form.
	 * 
	 * @param encodedPassword
	 *            Encoded form of the password
	 * @return <code>true</code> if the underlying password is blank.
	 */
	public static boolean isBlankPassword(String encodedPassword) {
		return StringUtils.equals(blankPassword, encodedPassword);
	}

	/**
	 * Program used to compute the digests of all password that are given on
	 * command line.
	 * 
	 * @param args
	 *            List of passwords.
	 */
	public static void main(String[] args) {
		for (int i = 0; i < args.length; i++) {
			String arg = args[i];
			char[] password = arg.toCharArray();
			String encoded = digestPassword(password);
			System.out.println(arg + " : " + encoded);
		}
	}

	/**
	 * Converts a character password to a byte password using utf8
	 * 
	 * @param password
	 *            Password to convert
	 * @return Password as bytes
	 */
	public static byte[] passwordToBytes(char[] password) {
		try {
			// Get the bytes for the characters
			ByteArrayOutputStream bytesOutput = new ByteArrayOutputStream();
			try {
				// Writes the chars
				OutputStreamWriter writer = new OutputStreamWriter(bytesOutput,
						"UTF-8");
				try {
					if (password != null) {
						writer.write(password);
					}
				} finally {
					writer.close();
				}
				// Get the bytes
				byte[] bytes = bytesOutput.toByteArray();
				return bytes;
			} finally {
				bytesOutput.reset();
			}
		} catch (Exception ex) {
			throw new LocalizableException(
					StringCodes.STRING_ERROR_CANNOT_ENCODE_PASSWORD, ex);
		}
	}

	private PasswordUtils() {
	}
}
